import os
import csv
import time
import pandas as pd
from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By

OUTPUT_FILE = 'minfin_books.csv'
START_URL = 'https://minfin.gov.ru/ru/ministry/historylib/'
WAIT_TIME = 3  # увеличьте до 5–10 если надо

USER_AGENT = (
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
    "AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/125.0.0.0 Safari/537.36"
)

# Если нужен прокси — впишите сюда свой адрес
PROXY = None
# Пример: PROXY = "http://username:password@host:port"

def init_browser():
    chrome_options = Options()
    chrome_options.add_argument("--headless=new")
    chrome_options.add_argument("--disable-gpu")
    chrome_options.add_argument("--window-size=1920,1080")
    chrome_options.add_argument("--no-sandbox")
    chrome_options.add_argument(f"user-agent={USER_AGENT}")
    if PROXY:
        chrome_options.add_argument(f'--proxy-server={PROXY}')
    return webdriver.Chrome(options=chrome_options)

def load_existing_entries():
    if not os.path.exists(OUTPUT_FILE):
        return set()
    df = pd.read_csv(OUTPUT_FILE)
    return set(df['source_link'].values)

def scroll_to_load_all(driver):
    last_height = driver.execute_script("return document.body.scrollHeight")
    while True:
        driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
        time.sleep(WAIT_TIME)
        new_height = driver.execute_script("return document.body.scrollHeight")
        if new_height == last_height:
            break
        last_height = new_height

def extract_books(driver, existing_links):
    soup = BeautifulSoup(driver.page_source, 'html.parser')
    tiles = soup.select('.history-tile')
    books = []
    for tile in tiles:
        try:
            title_tag = tile.select_one('.document-title a')
            if not title_tag:
                continue
            name = title_tag.text.strip()
            source_link = f"https://minfin.gov.ru{title_tag['href']}"
            if source_link in existing_links:
                continue
            pdf_button = tile.select_one('a[title*="Скачать"]')
            if not pdf_button:
                continue
            pdf_link = 'https://minfin.gov.ru' + pdf_button['href']
            books.append({
                'name': name,
                'source_link': source_link,
                'book_link': pdf_link
            })
        except Exception as e:
            print(f'Ошибка разбора плитки: {e}')
            continue
    return books

def append_to_csv(book_data):
    file_exists = os.path.isfile(OUTPUT_FILE)
    with open(OUTPUT_FILE, 'a', newline='', encoding='utf-8') as csvfile:
        fieldnames = ['name', 'source_link', 'book_link']
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
        if not file_exists:
            writer.writeheader()
        for row in book_data:
            writer.writerow(row)

def main():
    existing_links = load_existing_entries()
    print(f"Уже обработано записей: {len(existing_links)}")
    driver = init_browser()
    driver.get(START_URL)
    print("Парсим динамически загружаемые данные...")
    scroll_to_load_all(driver)
    print("Загрузка завершена. Извлечение данных...")
    new_books = extract_books(driver, existing_links)
    print(f"Найдено новых книг: {len(new_books)}")
    if new_books:
        append_to_csv(new_books)
        print(f"✅ Добавлено в CSV: {len(new_books)} записей")
    else:
        print("🔍 Новых книг не найдено — все актуально")
    driver.quit()

if __name__ == "__main__":
    try:
        main()
    except Exception as e:
        print(f"🚫 Ошибка выполнения скрипта: {e}")
